package hdm.mi.sthbackend.controller;


import hdm.mi.sthbackend.dto.PlayerDTO;
import hdm.mi.sthbackend.exeptions.*;
import hdm.mi.sthbackend.dto.TeamDTO;
import hdm.mi.sthbackend.model.Player;
import hdm.mi.sthbackend.model.Team;
import hdm.mi.sthbackend.model.Tournament;
import hdm.mi.sthbackend.service.TournamentService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.UUID;

@RestController
@RequestMapping("/api/v1")
@AllArgsConstructor
public class TournamentController {

    private final TournamentService service;

    /**
     * Player Endpoints
     **/
    @GetMapping("players/{playerId}")
    public PlayerDTO findPlayerById(@PathVariable UUID playerId) throws PlayerIdNotFoundException {
        return service.findPlayerById(playerId);
    }

    @PatchMapping("/players/{playerId}")
    public UUID updatePlayerName(@PathVariable UUID playerId,
                                 @RequestBody String newPlayerName) throws PlayerIdNotFoundException {
        return service.updatePlayerName(playerId, newPlayerName);
    }

    /**
     * Team Endpoints
     **/
    @GetMapping("/teams/{teamId}")
    public TeamDTO findTeamById(@PathVariable UUID teamId) throws TeamIdNotFoundException {
        return service.findTeamById(teamId);
    }

    @PostMapping("/tournaments/{tournamentId}/teams")
    public Team createTeam(@PathVariable UUID tournamentId,
                           @RequestBody String teamName) throws TournamentIdNotFoundException {
        return service.createAndAddTeamToTournament(tournamentId, teamName);
    }

    @DeleteMapping("/tournaments/{tournamentId}/teams/{teamId}")
    public UUID removeTeamFromTournament(@PathVariable UUID tournamentId,
                                         @PathVariable UUID teamId) throws TournamentIdNotFoundException, TeamIdNotFoundException {
        return service.deleteAndRemoveTeamFromTournament(tournamentId, teamId);
    }

    @PostMapping("/teams/{teamId}/addPlayer")
    public Player addPlayerToTeam(@PathVariable UUID teamId,
                                  @RequestBody String playerName) throws Exception {
        return service.addPlayerToTeam(teamId, playerName);
    }

    @DeleteMapping("/teams/{teamId}/players/{playerId}")
    public PlayerDTO removePlayerFromTeam(@PathVariable UUID teamId,
                                          @PathVariable UUID playerId) throws PlayerIdNotFoundException, TeamIdNotFoundException {
        return service.removePlayerFromTeam(teamId, playerId);
    }
    @PatchMapping("/tournaments/{tournamentId}/match/{matchId}")
    public UUID determineWinner(@PathVariable UUID tournamentId,
                                @PathVariable UUID matchId) throws TournamentIdNotFoundException, MatchIdNotFoundException, WinnerNotDeterminedException {
        return service.determineWinner(tournamentId, matchId);
    }


    /**
     * Tournament Endpoints
     */
    @PostMapping("/tournaments")
    public Tournament createTournament(@RequestBody String tournamentName) {
        return service.createTournament(tournamentName);
    }

}
